﻿using System;
using System.Reflection;
using CoreGraphics;
using Syncfusion.SfPdfViewer.iOS;
using UIKit;
using System.Diagnostics;
using System.IO;
using System.Collections.Generic;

namespace GettingStarted
{
	public partial class ViewController : UIViewController
	{

		SfPdfViewer pdfViewerControl;
		private const float DefaultToolbarHeight = 50f;

        private UIView parentView;
		private UIView toolbar;
		UITextField pageNumberField = new UITextField();
		UILabel totalPageLabel = new UILabel();
		UIButton previousPageBtn = new UIButton();
		UIButton nextPageBtn = new UIButton();
        UISearchBar searchBar = new UISearchBar();
        UIButton searchButton = new UIButton();
        UIButton backButton = new UIButton();
		UIButton searchNext = new UIButton();
		UIButton searchPrevious = new UIButton();
        Stopwatch stopwatch = new Stopwatch();
        UIButton fileButton = new UIButton();
        CGRect toolBarFrame = new CGRect();
        bool isLoaded = false;
        UIToolbar searchToolBar = new UIToolbar();
        UIToolbar toolBar = new UIToolbar();
        ActivityIndicator activityDialog;
        DropDownMenu dropDownMenu;
        Stream fileStream;
        UIAlertView popUpAlertView; 

        protected ViewController(IntPtr handle) : base(handle)
		{
			
		}

		public override void ViewDidLoad()
		{
			base.ViewDidLoad();
			parentView = new UIView(this.View.Frame);
			var tap = new UITapGestureRecognizer(() => this.View.EndEditing(true));
			tap.CancelsTouchesInView = false; 

			this.View.AddGestureRecognizer(tap);
			pdfViewerControl = new SfPdfViewer();
			pdfViewerControl.PageChanged += ViewerControl_PageChanged;
            pdfViewerControl.SearchInitiated += PdfViewerControl_SearchInitiated;
            pdfViewerControl.SearchCompleted += PdfViewerControl_SearchCompleted;
			pdfViewerControl.TextSearchSettings.CurrentInstanceColor = UIColor.FromRGBA(0, 0, 255, 120);
			pdfViewerControl.TextSearchSettings.OtherInstanceColor = UIColor.FromRGBA(0, 0, 255, 50);
            pageNumberField.Text = "1";
			toolbar = createTopToolbar();
			parentView.AddSubview(pdfViewerControl);
			this.View.AddSubview(parentView);
			this.View.AddSubview(toolbar);
            activityDialog = new ActivityIndicator();
            activityDialog.Frame = new CGRect(UIScreen.MainScreen.Bounds.Width / 2 - 125, UIScreen.MainScreen.Bounds.Height / 2 - 50, 250, 100);
            popUpAlertView = new UIAlertView();
            dropDownMenu = CreateDropDownMenu();

            dropDownMenu.DropDownMenuItemChanged += (e, a) =>
            {
                fileStream = typeof(ViewController).GetTypeInfo().Assembly.GetManifestResourceStream("GettingStarted.Assets." + a.DisplayText + ".pdf");
                pdfViewerControl.LoadDocument(fileStream);
                ResetToolBar();
                dropDownMenu.Close();
            };
        }

        private void PdfViewerControl_SearchCompleted(object sender, TextSearchEventArgs args)
        {
            stopwatch.Stop();
            if (!stopwatch.IsRunning)
            {
                if (activityDialog != null)
                {
                    activityDialog.StopAnimating();
                    activityDialog.RemoveFromSuperview();
                    if (args.NoMatchFound)
                    {
                        popUpAlertView.Title = "Search Result";
                        popUpAlertView.Message = "No matches found";
                        popUpAlertView.AddButton("OK");
                        popUpAlertView.Show();
                    }
                }
            }
        }

        private void PdfViewerControl_SearchInitiated(object sender, TextSearchEventArgs args)
        {
            if (stopwatch.ElapsedMilliseconds > 2000)
            {
                View.AddSubview(activityDialog);
                activityDialog.SetMessage("Search in progress...");
                activityDialog.StartAnimating();
            }
        }

        public override void DidReceiveMemoryWarning()
		{
			base.DidReceiveMemoryWarning();
			// Release any cached data, images, etc that aren't in use.
		}
		private void ViewerControl_PageChanged(object sender, PageChangedEventArgs args)
		{
			pageNumberField.Text = args.PageNumber.ToString();
		}

		public override void ViewDidLayoutSubviews()
		{
			base.ViewDidLayoutSubviews();
			parentView.Frame = new CGRect(this.View.Frame.X,DefaultToolbarHeight+20, this.View.Frame.Width, this.View.Frame.Height-DefaultToolbarHeight-20);
		}

		public override void ViewDidAppear(bool animated)
		{
			base.ViewDidAppear(animated);
			var fileStream = typeof(ViewController).GetTypeInfo().Assembly.GetManifestResourceStream("GettingStarted.Assets.GIS Succinctly.pdf");
			pdfViewerControl.LoadDocument(fileStream);
			totalPageLabel.Text = "/ " + pdfViewerControl.PageCount.ToString();

		}

		protected virtual UIView createTopToolbar()
		{
			var toolBarFrame = new CGRect();
			toolBarFrame = this.View.Frame;
			toolBarFrame.Height = DefaultToolbarHeight;
			toolBarFrame.Y = 20;
			toolBar.Frame = toolBarFrame;

			toolBar.AutoresizingMask = UIViewAutoresizing.FlexibleBottomMargin | UIViewAutoresizing.FlexibleWidth;

            fileButton.Frame = new CGRect(10, 5, 40, 40);
            fileButton.HorizontalAlignment = UIControlContentHorizontalAlignment.Center;
            fileButton.TouchUpInside += (e, a) =>
            {
                if (dropDownMenu.IsOpened)
                    dropDownMenu.Close();
                else
                    dropDownMenu.Open();
            };
            fileButton.SetBackgroundImage(UIImage.FromFile("FileIcon.png"), UIControlState.Normal);
            toolBar.Add(fileButton);

            pageNumberField.KeyboardType = UIKeyboardType.NumberPad;
            pageNumberField.EditingDidBegin += PageNumberField_EditingDidBegin;
            pageNumberField.ShouldReturn += (textField) =>
            {
                textField.ResignFirstResponder();
                int input = 0;
                if (int.TryParse(pageNumberField.Text, out input) && input > 0 && (input <= pdfViewerControl.PageCount))
                {
                    pdfViewerControl.GoToPage(input);
                }
                else
                {
                    UIAlertView alert = new UIAlertView();
                    alert.Title = "Error";
                    alert.AddButton("OK");
                    alert.Message = "Please enter the valid page number";
                    alert.Show();
                    pageNumberField.Text = pdfViewerControl.PageNumber.ToString();
                }
                pageNumberField.Text = pdfViewerControl.PageNumber.ToString();
                return true;
            };

            pageNumberField.BorderStyle = UITextBorderStyle.RoundedRect;
            pageNumberField.Frame = new CGRect(65, 5, 45, 40);
            pageNumberField.HorizontalAlignment = UIControlContentHorizontalAlignment.Center;
            pageNumberField.TextAlignment = UITextAlignment.Center;
            toolBar.Add(pageNumberField);


            totalPageLabel.Frame = new CGRect(120, 5, 40, 40);
            toolBar.Add(totalPageLabel);

            nextPageBtn.HorizontalAlignment = UIControlContentHorizontalAlignment.Center;
            nextPageBtn.Frame = new CGRect(175, 5, 40, 40);
            nextPageBtn.HorizontalAlignment = UIControlContentHorizontalAlignment.Center;
            nextPageBtn.TouchUpInside += HandleTouchDownInside;
            nextPageBtn.SetBackgroundImage(UIImage.FromFile("PageDown.png"), UIControlState.Normal);
            toolBar.Add(nextPageBtn);

            previousPageBtn.Frame = new CGRect(230, 5, 40, 40);
            previousPageBtn.HorizontalAlignment = UIControlContentHorizontalAlignment.Center;
            previousPageBtn.TouchUpInside += HandleTouchUpInside;
            previousPageBtn.SetBackgroundImage(UIImage.FromFile("PageUp.png"), UIControlState.Normal);
            toolBar.Add(previousPageBtn);


            searchButton.Frame = new CGRect(parentView.Frame.Width - 50, 5, 40, 40);
            searchButton.HorizontalAlignment = UIControlContentHorizontalAlignment.Center;
            searchButton.TouchUpInside += SearchButtonClicked;
            searchButton.SetBackgroundImage(UIImage.FromFile("Search.png"), UIControlState.Normal);
            toolBar.Add(searchButton);

            return toolBar;
        }

        private void PageNumberField_EditingDidBegin(object sender, EventArgs e)
        {
            pageNumberField.ClearsOnBeginEditing = true;
        }

        protected virtual UIView CreateSearchTopToolbar()
        {
            toolBarFrame = View.Frame;
            toolBarFrame.Height = DefaultToolbarHeight;
            toolBarFrame.Y = 20;
            searchToolBar.Frame = toolBarFrame;

            searchToolBar.AutoresizingMask = UIViewAutoresizing.FlexibleBottomMargin | UIViewAutoresizing.FlexibleWidth;

            if ((UIDevice.CurrentDevice).UserInterfaceIdiom == UIUserInterfaceIdiom.Pad)
                backButton.Frame = new CGRect(20, 2, 50, 50);
            else
                backButton.Frame = new CGRect(2, 5, 40, 40);

            backButton.HorizontalAlignment = UIControlContentHorizontalAlignment.Center;
            backButton.TouchUpInside += BackButtonClicked;
            backButton.SetBackgroundImage(UIImage.FromFile("Back.png"), UIControlState.Normal);

            searchToolBar.Add(backButton);
            if ((UIDevice.CurrentDevice).UserInterfaceIdiom == UIUserInterfaceIdiom.Pad)
                searchBar.Frame = new CGRect(95, 5, 270, 40);
            else
                searchBar.Frame = new CGRect(40, 5, 140, 40);

            searchBar.Placeholder = "Enter text to search";
            searchBar.TextChanged += SearchBar_TextChanged;
            searchBar.SearchButtonClicked += SearchBar_SearchButtonClicked;
            searchToolBar.Add(searchBar);


			searchNext.Frame = new CGRect(415, 5, 40, 40);
            searchNext.HorizontalAlignment = UIControlContentHorizontalAlignment.Center;
			searchNext.TouchUpInside += SearchNextClicked;
            searchNext.SetBackgroundImage(UIImage.FromFile("SearchNext.png"), UIControlState.Normal);
            searchToolBar.Add(searchNext);

			searchPrevious.Frame = new CGRect(470, 5, 40, 40);
            searchPrevious.HorizontalAlignment = UIControlContentHorizontalAlignment.Center;
            searchPrevious.TouchUpInside += SearchPreviousClicked;
            searchPrevious.SetBackgroundImage(UIImage.FromFile("SearchPrevious.png"), UIControlState.Normal);
            searchToolBar.Add(searchPrevious);

            return searchToolBar;
        }

        private void SearchBar_TextChanged(object sender, UISearchBarTextChangedEventArgs e)
        {
            if ((sender as UISearchBar).Text == string.Empty)
            {
                pdfViewerControl.CancelSearch();
            }
        }

        private void SearchCancelClicked(object sender, EventArgs e)
        {
            if (activityDialog != null && activityDialog.IsAnimating)
            {
                activityDialog.StopAnimating();
                activityDialog.RemoveFromSuperview();
            }
            pdfViewerControl.CancelSearch();
            searchBar.Text = String.Empty;
        }

        private void SearchBar_SearchButtonClicked(object sender, EventArgs e)
        {
            searchBar.ResignFirstResponder();
            pdfViewerControl.SearchText(searchBar.Text);
        }

        void SearchClicked(object sender, EventArgs ea)
        {
            pdfViewerControl.SearchText(searchBar.Text);
        }

        void SearchNextClicked(object sender, EventArgs ea)
        {
            stopwatch.Start();
            pdfViewerControl.SearchNext(searchBar.Text);
        }

        void SearchPreviousClicked(object sender, EventArgs ea)
        {
            stopwatch.Start();
            pdfViewerControl.SearchPrevious(searchBar.Text);
        }

        void SearchButtonClicked(object sender, EventArgs ea)
        {
            toolbar.RemoveFromSuperview();
            backButton.Enabled = true;
            CreateSearchTopToolbar();
            toolbar = searchToolBar;
            searchBar.BecomeFirstResponder();
            View.AddSubview(toolbar);
        }

        void BackButtonClicked(object sender, EventArgs ea)
        {
            if (activityDialog != null && activityDialog.IsAnimating)
            {
                activityDialog.StopAnimating();
                activityDialog.RemoveFromSuperview();
            }
            pdfViewerControl.CancelSearch();
            searchBar.Text = String.Empty;
            UpdateToolBarValues();
            searchToolBar.RemoveFromSuperview();
            toolBar.RemoveFromSuperview();
            toolbar = toolBar;
            View.AddSubview(toolbar);
        }

        public void UpdateToolBarValues()
        {
            pageNumberField.Text = pdfViewerControl.PageNumber.ToString();

        }

        void HandleTouchDownInside(object sender, EventArgs ea)
		{
			pdfViewerControl.GoToNextPage();
		}

		void HandleTouchUpInside(object sender, EventArgs ea)
		{
			pdfViewerControl.GoToPreviousPage();
		}

        private DropDownMenu CreateDropDownMenu()
        {
            var dropDownMenu = new DropDownMenu(this.View, GetResource().ToArray())
            {
                BackgroundColor = UIColor.Clear,
                TextColor = UIColor.Black,
                Opacity = 1,
                BorderWidth = 1,
                Frame = new CGRect(0, DefaultToolbarHeight + 20, 250, 200)
            };
            return dropDownMenu;
        }

        private List<DropDownMenuItem> GetResource()
        {

            var list = new List<DropDownMenuItem>();
            list.Add(new DropDownMenuItem()
            {
                DisplayText = "F# Succinctly",
            });
            list.Add(new DropDownMenuItem()
            {
                DisplayText = "GIS Succinctly",
                IsSelected = true
            });
            list.Add(new DropDownMenuItem()
            {
                DisplayText = "HTTP Succinctly",
            });
            list.Add(new DropDownMenuItem()
            {
                DisplayText = "JavaScript Succinctly",
            });
            return list;
        }

        private void ResetToolBar()
        {
            pageNumberField.Text = pdfViewerControl.PageNumber.ToString();
            totalPageLabel.Text = "/ " + pdfViewerControl.PageCount.ToString();
        }
    }
}
